# 项目规范
## 前言 

> 本文档用于制定项目的统一设计，开发，管理规范。内容可能不主流，但制定的目的在于使得项目对内统一。

## 安全规约
1. 【强制】隶属于用户个人的页面或者功能必须进行权限控制校验。
    - 说明：防止没有做水平权限校验就可随意访问、修改、删除别人的数据，比如查看他人的私信内容。

2. 【强制】用户敏感数据禁止直接展示，必须对展示数据进行脱敏。
    - 说明：中国大陆个人手机号码显示：139****1219，隐藏中间 4 位，防止隐私泄露。
3. 【强制】用户输入的 SQL 参数严格使用参数绑定或者 METADATA 字段值限定，防止 SQL 注入，禁止字符串拼接 SQL 访问数据库。
    - 反例：某系统签名大量被恶意修改，即是因为对于危险字符 # --没有进行转义，导致数据库更新时，where后边的信息被注释掉，对全库进行更新。
4. 【强制】用户请求传入的任何参数必须做有效性验证。
    - 说明：忽略参数校验可能导致：
        + page size 过大导致内存溢出
        + 恶意 order by 导致数据库慢查询
        + 缓存击穿
        + SSRF
        + 任意重定向
        + SQL 注入，Shell 注入，反序列化注入
        + 正则输入源串拒绝服务 ReDoS
    - Java 代码用正则来验证客户端的输入，有些正则写法验证普通用户输入没有问题，但是如果攻击人员使用
    的是特殊构造的字符串来验证，有可能导致死循环的结果。

5. 【强制】在使用平台资源，譬如短信、邮件、电话、下单、支付，必须实现正确的防重放的机制，如数量限制、疲劳度控制、验证码校验，避免被滥刷而导致资损。
    - 说明：如注册时发送验证码到手机，如果没有限制次数和频率，那么可以利用此功能骚扰到其它用户，并造成短信平台资源浪费。


## 前后端规约
1. 【强制】前后端交互的 API，需要明确协议、域名、路径、请求方法、请求内容、状态码、响应体。
    - 说明：
        + 1） 路径：每一个 API 需对应一个路径，表示 API 具体的请求地址：
            * a） 代表一种资源，只能为名词，推荐使用复数，不能为动词，请求方法已经表达动作意义。
            * b） URL 路径不能使用大写，单词如果需要分隔，统一使用下划线。
            * c） 路径禁止携带表示请求内容类型的后缀，比如".json",".xml"，通过 accept 头表达即可。
        + 2） 请求方法：对具体操作的定义，常见的请求方法如下：
            * a） GET：从服务器取出资源。
            * b） POST：在服务器新建一个资源。
            * c） PUT：在服务器更新资源。
            * d） DELETE：从服务器删除资源。
        + 3） 请求内容：URL 带的参数必须无敏感信息或符合安全要求；

2.【强制】前后端数据列表相关的接口返回，如果为空，则返回空数组[]或空集合{}。
   - 说明：此条约定有利于数据层面上的协作更加高效，减少前端很多琐碎的 null 判断

3.【强制】服务端发生错误时，返回给前端的响应信息必须包含 HTTP 状态码，errorCode、errorMessage、用户提示信息四个部分。
   - 说明：四个部分的涉众对象分别是浏览器、前端开发、错误排查人员、用户。<br/>
    errorCode：[参见errorCode](/doc/errorCode.md)。<br/>
    errorMessage： 简要描述后端出错原因，便于错误排查人员快速定位问题，注意不要包含敏感数据信息。<br/>
    用户的提示信息要求：简短清晰、提示友好，引导用户进行下一步操作或解释错误原因，提示信息可以包括错误原因、上下文环境、推荐操作等。<br/>

4.【强制】在前后端交互的 JSON 格式数据中，所有的 key 必须为小写字母开始的lowerCamelCase 风格，符合英文表达习惯，且表意完整。
  - 正例：errorCode / errorMessage / assetStatus / menuList / orderList / configFlag
  - 反例：ERRORCODE / ERROR_CODE / error_message / error-message / errormessage / ErrorMessage / msg

5.【强制】对于需要使用超大整数的场景，服务端一律使用 String 字符串类型返回，禁止使用 Long 类型。

6.【强制】前后端的时间格式统一为"yyyy-MM-dd HH:mm:ss"，统一为 GMT。

7.【强制】前后端的金钱格式统一为分。

8.【强制】前后端交换数据涉及到不同类型时使用字符串来代替传统的int。
    - 正例 RequestBody：{"payType":"PAY_BY_WECHAT","orderCode":"123"}
    - 反例 RequestBody：{"payType":1,"orderCode":"123"}
    

## 后端规约
1.【强制】在引用依赖时想清楚我要使用什么，它能给我什么，它里面有什么
    - 说明：防止后期出现依赖冲突的问题

2.【强制】禁止向外依赖
    - 说明：DDD中每一层都是设计好的，不允许出现向外依赖的情况

3.【强制】不允许使用JSON.toJSONString来作为日志打印对象
    - 说明：JSON.toJSONString是采用反射getter的方式来转换成字符串的，这可能会存在一些问题，我们应当自己重写toString

4.【推荐】类成员与方法访问控制从严，对外部提供最小权限访问

5.【强制】异常处理以提早抛出，延迟捕获为原则。异常信息应该包括两类信息：案发现场信息和异常堆栈信息。如果不处理，那么通过关键字 throws 往上抛出。

6.【推荐】如果不能使用英文表达的内容请使用中文，以免带来理解难度和歧义

7.【推荐】对于核心逻辑编写单元测试

## 前端规约
1